/* wmDrawer (c) 2002-2003 Valery Febvre <vfebvre@lautre.net>
 *
 * wmDrawer is a dock application (dockapp) which provides a
 * drawer (button bar) to launch applications from.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef USE_GDKPIXBUF
#include <gdk-pixbuf/gdk-pixbuf-xlib.h>
#endif

#ifdef USE_GDKPIXBUF2
#include <gdk-pixbuf-xlib/gdk-pixbuf-xlib.h>
#endif

#include "types_defs.h"
#include "images.h"

unsigned int gdkPixbufIsInit = 0;
extern Display *display;
extern drawerConfig config;
extern int nbImagesPaths;

static GdkPixbuf *loadImageFromFile(const char *file) {
#ifdef USE_GDKPIXBUF
  return gdk_pixbuf_new_from_file(file);
#endif
#ifdef USE_GDKPIXBUF2
  GError *error = NULL;
  return gdk_pixbuf_new_from_file(file, &error);
#endif
}

int createPixmapFromFile (const char *file, Pixmap *imgPix, Pixmap *imgMask, int *w, int *h, int maxDim) {
  GdkPixbuf *img = NULL, *imgScaled = NULL;
  int i, imgW, imgH, imgScaledW, imgScaledH;
  char path[FILENAME_MAX];

  if (!gdkPixbufIsInit) {
#ifdef USE_GDKPIXBUF2
    g_type_init ();
#endif
    gdk_pixbuf_xlib_init (display, DefaultScreen (display));
    gdkPixbufIsInit = 1;
  }

  /* quick test for absolute path */
  if (file[0] != '/' || (img = loadImageFromFile (file)) == NULL) {
    /* else try all paths */
    for (i=0; i<nbImagesPaths; i++) {
      sprintf(path, "%s/%s", config.imagesPaths[i], file);
      img = loadImageFromFile (path);
      if (img != NULL) break;
    }
  }

  if (img == NULL) {
    printf ("%s warning: can't load image %s\n", PACKAGE, file);
    return 0;
  }

  imgW = gdk_pixbuf_get_width (img);
  imgH = gdk_pixbuf_get_height (img);
  *w = imgW;
  *h = imgH;
  /* resize smaller if necessary */
  if (imgW > maxDim || imgH > maxDim) {
    if (imgW >= imgH) {
      imgScaledW = maxDim;
      imgScaledH = (int) ((imgH * maxDim) / imgW);
    }
    else {
      imgScaledW = (int) ((imgW * maxDim) / imgH);
      imgScaledH = maxDim;
    }
    *w = imgScaledW;
    *h = imgScaledH;
    imgScaled = gdk_pixbuf_scale_simple (img, imgScaledW, imgScaledH, GDK_INTERP_HYPER);
    gdk_pixbuf_unref (img); /* Deprecated with gdk-pixbuf-2.0 */
    /* g_object_unref(img); */
    img = imgScaled;
  }
  gdk_pixbuf_xlib_render_pixmap_and_mask (img, imgPix, imgMask, 128);
  gdk_pixbuf_unref (img); /* Deprecated with gdk-pixbuf-2.0 */
  /* g_object_unref(img); */ 
  return 1;
}

void createPixmapFromData (const char **data, Pixmap *imgPix, Pixmap *imgMask, int *w, int *h, int maxDim) {
  GdkPixbuf *img, *imgScaled;
  int imgW, imgH, imgScaledW, imgScaledH;

  if (!gdkPixbufIsInit) {
#ifdef USE_GDKPIXBUF2
    g_type_init ();
#endif
    gdk_pixbuf_xlib_init (display, DefaultScreen (display));
    gdkPixbufIsInit = 1;
  }

  img = gdk_pixbuf_new_from_xpm_data ((const char **)data);

  imgW = gdk_pixbuf_get_width (img);
  imgH = gdk_pixbuf_get_height (img);
  *w = imgW;
  *h = imgH;
  /* resize smaller if necessary */
  if (imgW > maxDim || imgH > maxDim) {
    if (imgW >= imgH) {
      imgScaledW = maxDim;
      imgScaledH = (int) ((imgH * maxDim) / imgW);
    }
    else {
      imgScaledW = (int) ((imgW * maxDim) / imgH);
      imgScaledH = maxDim;
    }
    *w = imgScaledW;
    *h = imgScaledH;
    imgScaled = gdk_pixbuf_scale_simple (img, imgScaledW, imgScaledH, GDK_INTERP_HYPER);
    gdk_pixbuf_unref (img);
    img = imgScaled;
  }
  gdk_pixbuf_xlib_render_pixmap_and_mask (img, imgPix, imgMask, 127);
  gdk_pixbuf_unref (img);
}
